function [time, lon, lat, T, T_sige, T_mean, T_seas, T_seas_sige, T_res, T_res_sige] = OH14_loadmap(date_start, date_end, lon_W, lon_E, lat_S, lat_N, path_to_OH14_nc, netcdf_version, path_to_netcdf_toolbox)

%
% [time, lon, lat, T, T_sige, T_mean, T_seas, T_seas_sige, T_res, T_res_sige] = OH14_loadmap(date_start, date_end, lon_W, lon_E, lat_S, lat_N, path_to_OH14_nc, netcdf_version, path_to_netcdf_toolbox)
% 
% Load data cube (time x latitude x longitude) from Oliver and Holbrook (2014)
% downscaled continental shelf sea surface temperature dataset. User specifies 
% start and end dates and longitude and latitude limits, function returns mean
% SST, SST seasonal cycle, residual SST and total SST within specified domain.
% Also ouputs error estimate on mean, seasonal cycle and residual SST. Units of
% SST are degrees Celcius.
%
% Inputs:
%
%  date_start      - start date (>= 14 Oct 1992), [year month day]
%  date_end        - end date (<=13 May 2008), [year month day]
%                    (if date_end = date_start then just returns a single day)
%  lon_W           - westernmost longitude, scalar
%  lon_E           - easternmost longitude, scalar
%  lat_S           - southernmost latitude, scalar
%  lat_N           - northermost latitude, scalar
%  path_to_OH14_nc - path to OH14 netcdf file, character string
%  netcdf_version  - which implementation of netcdf to use, character string:
%                    'matlab_api': built-in MATLAB netcdf implementation
%                                  (Requires MATLAB 2010a or later) [DEFAULT]
%                    'netcdf_toolbox': adds external 
%  path_to_netcdf_toolbox - path to folder containing netcdf_toolbox and mexcdf
%                           libraries (only required if netcdf_version = 'netcdf_toolbox'
%                     
% Outputs:
%
%  time            - time vector (in MATLAB datenum format), [TIME, 1]
%  lon, lat        - longitude and latitude vectors, [X, 1] and [Y, 1]
%  T               - SST (total = mean + seasonal cycle + residual) [TIME, X, Y]
%  T_sige          - Error on SST, given as a standard deviation [X, Y]
%  T_mean          - Mean SST [X, Y]
%  T_seas          - SST seasonal cycle [TIME, X, Y]
%  T_seas_sige     - Error on SST seasonal cycle, given as a standard deviation [X, Y]
%  T_res           - SST residual [TIME, X, Y]
%  T_res_sige      - Error on SST residual, given as a standard deviation [X, Y]
%
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
%
% Created 3-7 May 2014 by Eric Oliver
%
% Eric Oliver
% Institute for Marine and Antarctic Studies
% University of Tasmania, Hobart TAS, Australia
% office: (+61) 6226 6663
% email: eric.oliver@utas.edu.au
% homepage: http://passage.phys.ocean.dal.ca/~olivere/
%

% Check if netcdf_toolbox requested
  if nargin < 8
    netcdf_version = 'matlab_api';
  end
  if ~(strcmp(netcdf_version, 'matlab_api') | strcmp(netcdf_version, 'netcdf_toolbox'))
      error('netcdf_version must be either ''matlab_api'' or ''netcdf_toolbox''')
  end

% Load latitude, longitude and time vectors
  if strcmp(netcdf_version, 'matlab_api')
    time_full = ncread(path_to_OH14_nc, 'time');
    lon_map = ncread(path_to_OH14_nc, 'lon_map');
    lat_map = ncread(path_to_OH14_nc, 'lat_map');
  else
    addpath(genpath(path_to_netcdf_toolbox)); % add netcdf_toolbox and mexcdf libraries
    nc = netcdf(path_to_OH14_nc, 'nowrite');
    time_full = nc{'time'};
    lon_map = nc{'lon_map'};
    lat_map = nc{'lat_map'};
  end

% Load indices of shelf data
  if strcmp(netcdf_version, 'matlab_api')
    is = ncread(path_to_OH14_nc, 'is');
    js = ncread(path_to_OH14_nc, 'js');
  else
    is = nc{'is'};
    js = nc{'js'};
  end

% Load scale factor and add offset
  if ~strcmp(netcdf_version, 'matlab_api')
    scale_factor = nc{'T_mean'}.scale_factor(:);
    add_offset = nc{'T_mean'}.add_offset(:);
  end

% Find indices of latitude and longitude
  i_W = find((lon_map >= lon_W) & (lon_map <= lon_E), 1, 'first');
  i_E = find((lon_map >= lon_W) & (lon_map <= lon_E), 1, 'last');
  j_S = find((lat_map >= lat_S) & (lat_map <= lat_N), 1, 'first');
  j_N = find((lat_map >= lat_S) & (lat_map <= lat_N), 1, 'last');

% Find time indices
  tt_start = find(time_full==datenum(date_start));
  tt_end = find(time_full==datenum(date_end));
  tt_span = length(tt_start:tt_end);

% Construct output matrices
  lon = lon_map(i_W:i_E);
  lat = lat_map(j_S:j_N);
  time = time_full(tt_start:tt_end);

  X = length(lon);
  Y = length(lat);
  TIME = length(time);

  T = NaN*ones(TIME,Y,X);
  T_mean = NaN*ones(Y,X);
  T_seas = NaN*ones(TIME,Y,X);
  T_res = NaN*ones(TIME,Y,X);

  T_sige = NaN*ones(Y,X);
  T_seas_sige = NaN*ones(Y,X);
  T_res_sige = NaN*ones(Y,X);

% Loop over latitude and longitude and load data into output matrices
  ii = 0;
  for i = i_W:i_E
    ii = ii + 1;
    jj = 0;
    for j = j_S:j_N
      jj = jj + 1;
% Find index
      p = find((is == i) & (js == j));
% Check if shelf data exists at this location
      if length(p) == 0, continue, end
% Load and assign values to output matrices
      if strcmp(netcdf_version, 'matlab_api')
        T(:,jj,ii) = ncread(path_to_OH14_nc, 'T', [p tt_start], [1 tt_span], [1 1]);
        T_mean(jj,ii) = ncread(path_to_OH14_nc, 'T_mean', [p], [1], [1]);
        T_seas(:,jj,ii) = ncread(path_to_OH14_nc, 'T_seas', [p tt_start], [1 tt_span], [1 1]);
        T_res(:,jj,ii) = ncread(path_to_OH14_nc, 'T_res', [p tt_start], [1 tt_span], [1 1]);
        T_sige(jj,ii) = ncread(path_to_OH14_nc, 'T_sige', [p], [1], [1]);
        T_seas_sige(jj,ii) = ncread(path_to_OH14_nc, 'T_seas_sige', [p], [1], [1]);
        T_res_sige(jj,ii) = ncread(path_to_OH14_nc, 'T_res_sige', [p], [1], [1]);
      else
        T(:,jj,ii) = nc{'T'}(tt_start:tt_end,p)*scale_factor + add_offset;
        T_mean(jj,ii) = nc{'T_mean'}(p)*scale_factor + add_offset;
        T_seas(:,jj,ii) = nc{'T_seas'}(tt_start:tt_end,p)*scale_factor + add_offset;
        T_res(:,jj,ii) = nc{'T_res'}(tt_start:tt_end,p)*scale_factor + add_offset;
        T_sige(jj,ii) = nc{'T_sige'}(p)*scale_factor + add_offset;
        T_seas_sige(jj,ii) = nc{'T_seas_sige'}(p)*scale_factor + add_offset;
        T_res_sige(jj,ii) = nc{'T_res_sige'}(p)*scale_factor + add_offset;
      end
    end
  end

% Close netcdf file
  if strcmp(netcdf_version, 'netcdf_toolbox');
    close(nc);
    rmpath(genpath(path_to_netcdf_toolbox)); % remove netcdf_toolbox and mexcdf libraries
  end
